==> p5-MyObject-1.0.tgz: lib/perl5/site_perl/5.005/i386-freebsd/MyObject.pm <==
#!/usr/bin/perl -w

=head1 NAME

MyObject - Store Perl objects in a MySql database

=head1 VERSION

This document refers to version 1.0 of MyObject, released Nov 10, 1999.
Download MyObject.pm from http://www.frontierworld.com/perl/MyObject.pm

=head1 SYNOPSIS

First create a table Foo in a MySql database Bar:

	create table Foo
	(
		id		INTEGER UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		name		VARCHAR(255) NOT NULL,
		email		VARCHAR(255),
		telephone	VARCHAR(255),
		timestamp	TIMESTAMP
	);

Then create a Perl program to access table Foo in database Bar as objects:

	package Foo;
	@ISA = qw(MyObject);
	use MyObject;

	package main;
	Foo->connect(database=>'Bar', user=>'fred', password=>'bloggs');
	Foo->fields( qw(name email telephone) );
	my $foo = Foo->new(name=>'kev', email=>'kev@frontierworld.com');
	$foo->insert();
	$foo->{telephone} = '+44 207 359 0985';
	$foo->update();
	$foo = Foo->select('name=?', 'kev');
	print $foo->{name} . ' has email address ' . $foo->{email} . '\n';
	$foo->delete();
	Foo->disconnect();

=head1 DESCRIPTION

MyObject stores objects in a MySql database. It should be inherited by a class
wishing to persist its objects in the database. The inheriting class should
correspond to a table of the same name (case-sensitive) in the database. This
table must have an auto_increment column called "id" as its primary key.

MyObject provides database connectivity via methods connect() and disconnect(),
object creation via methods new(), select() and next(), and object/database
syncronisation via methods insert(), update() and delete().

MyObject does not check whether objects have been modified in the database while
they were being modified in your Perl program. Nor does it support any locking
mechanisms. Basically it's designed to be fast, simple, easy and predictable.

=cut
package MyObject;
$VERSION = "1.0";

use strict;
use Carp;
use DBI;
{
	# Class static properties - each is a hash keyed by class/table name

	my $_Connections = {};	# Database connections

	my $_Fields  = {};	# Fields to be syncronised with the database
	my $_Queries = {};	# User-defined queries for select() and next()

	my $_Selects = {};	# Standard select queries for row()
	my $_Inserts = {};	# Standard insert queries for insert()/update()
	my $_Updates = {};	# Standard update queries for update()
	my $_Deletes = {};	# Standard delete queries for delete()

	# Error messages

	my $_ERROR_FIELDS = "call class static method fields() first";

=head2 Class Methods

=over 4

=item connect(driver=>'mysql', database='dbname', user=>'username', password=>'pass', fields=>[qw(fields1 field2 field3)])

Connect to the database with optional connection details, and return the DBI
connection. The database driver always defaults to "mysql" unless environment
variable $DRIVER has been set. The database, user and password will default
to environment variables $DATABASE, $USER and $PASSWORD. If a field list is
not supplied, then a separate call to fields() must be made after connecting.

=cut
sub connect
{
	my ($self, %arg) = @_;
	my $class = ref($self) || $self;

	# Get the connection details

	my $driver	= $arg{driver}		|| $ENV{DRIVER} || "mysql";
	my $database	= $arg{database}	|| $ENV{DATABASE};
	my $user	= $arg{user}		|| $ENV{USER};
	my $password	= $arg{password}	|| $ENV{PASSWORD};

	# Connect to the database

	$_Connections->{$class} ||=
			DBI->connect("DBI:$driver:$database", $user, $password);

	# Create standard queries for the database fields

	$class->fields(@{ $arg{fields} }) if $arg{fields};

	# Return the database connection

	return $_Connections->{$class}
			or carp("cannot connect to database: $DBI::errstr");
}

=item disconnect()

Disconnect from the database cleanly. You must call disconnect() before calling
connect() for a second time, otherwise connect() will merely return a handle to
the current database connection.

=cut
sub disconnect
{
	my ($self) = @_;
	my $class = ref($self) || $self;

	# Finish user-defined queries

	foreach my $query (keys %{ $_Queries->{$class} })
	{
		$_Queries->{$class}->{$query}->finish();
	}

	# Finish standard queries

	$_Selects->{$class}->finish();
	$_Deletes->{$class}->finish();
	$_Inserts->{$class}->finish();
	$_Updates->{$class}->finish();

	# Close the database connection

	$_Connections->{$class}->disconnect()
		or carp("cannot disconnect from database: $DBI::errstr");

	$_Connections->{$class} = undef;
}

=item fields('field1', 'field2')

Setup a list of fields to keep in sync with the database. Do not include the
"id" field or any timestamp fields in the list. Only include fields that you
want to keep syncronised with the database. Call fields() immediately after
calling connect() if you didn't provide a field list when you called connect().

=cut
sub fields
{
	my ($self, @fields) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} = [@fields];

	my $dbh = $_Connections->{$class} || $class->connect();

	# Prepare standard queries

	$_Selects->{$class} = $dbh->prepare("select * from $class where id = ?");
	$_Deletes->{$class} = $dbh->prepare("delete from $class where id = ?");

	my $field_list = "";
	my $value_list = "";
	my $update_list = "";
	foreach my $field (@fields)
	{
		$field_list .= ", " if $field_list;
		$field_list .= $field;

		$value_list .= ", " if $value_list;
		$value_list .= "?";

		$update_list .= ", " if $update_list;
		$update_list .= "$field = ?";
	}

	$_Inserts->{$class} = $dbh->prepare("insert into $class ($field_list) values ($value_list)");
	$_Updates->{$class} = $dbh->prepare("update $class set $update_list where id = ?");
}

=item new(field1=>'value1', field2=>'value2')

Create a new object, setting any field values that were passed. This call does
not store the new object in the database - you should call insert() or update().

=cut
sub new
{
	my ($self, %arg) = @_;
	my $class = ref($self) || $self;
	bless { %arg }, $class;
}

=item row(id)

Get a table row as an object, by requesting a specific id. This is very useful
when one table has a relationship with another table, implemented via foreign
keys, and you wish to get the related object.

=cut
sub row
{
	my ($self, $id) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or carp($_ERROR_FIELDS);

	# Get an object with a specific id from the database

	$_Selects->{$class}->execute($id);
	$self = $_Selects->{$class}->fetchrow_hashref() || {};

	bless $self, $class;
}

=item select('field=?', 'value')

Select the first object matching some conditions. The format of the conditions
and arguments is similar to "printf", in that a "?" is used as a placeholder
for a corresponding argument. For example, if you use three question marks, you
should provide three arguments to instantiate them. Use next() to iterate.

=cut
sub select
{
	my ($self, $conditions, @args) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or carp($_ERROR_FIELDS);
	$conditions = "1" unless $conditions;

	# Create a user-defined query unless it's cached

	my $dbh = $_Connections->{$class};
	$_Queries->{$class}->{$conditions} ||=
			$dbh->prepare("select * from $class where $conditions");

	# Create a new object by executing the query

	my $query = $_Queries->{$class}->{$conditions};
	$query->execute(@args);
	$self = $query->fetchrow_hashref() || {};

	bless $self, $class;
}

=item next('field=?')

Select the next object matching some conditions. The conditions must exactly
match those originally provided in a select() call. When next() cannot return
the next object, it returns an empty object. A typical use for select() and
next() is in a for loop like this:

	my $obj;
	for (	$obj = Class->select("field=?",value);
		$obj->{id};
		$obj = Class->next("field=?"))
	{
		# Do something with $obj
	}

=cut
sub next
{
	my ($self, $conditions) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or carp($_ERROR_FIELDS);
	$conditions = "1" unless $conditions;

	# Get the next object from a user-defined query

	my $query = $_Queries->{$class}->{$conditions}
		or carp("select query not definied for $class: $conditions");

	$self = $query->fetchrow_hashref() || {};

	bless $self, $class;
}

=back

=head2 Object Methods

=over 4

=item insert()

Insert an object into the database and give the object a new id.

=cut
sub insert
{
	my ($self) = @_;
	$self->{id} = undef;
	$self->update();
}

=item update()

Update an object in the database, or insert it if the object has no id.

=cut
sub update
{
	my ($self) = @_;
	my $class = ref($self);
	$_Fields->{$class} or carp($_ERROR_FIELDS);

	# Get any object properties that are field values

	my @args = ();
	foreach my $field (@{ $_Fields->{$class} })
	{
		push @args, $self->{$field};
	}

	if ($self->{id})
	{
		# Update the object in the database

		push @args, $self->{id};
		$_Updates->{$class}->execute(@args);
	}
	else
	{
		# Insert the object into the database

		$_Inserts->{$class}->execute(@args);
		$self->{id} = $_Inserts->{$class}->{insertid};
	}
}

=item delete()

Delete an object from the database.

=cut
sub delete
{
	my ($self) = @_;
	return unless $self->{id};
	my $class = ref($self);
	$_Fields->{$class} or carp($_ERROR_FIELDS);

	# Delete an object from the database and give it an undefined id

	$_Deletes->{$class}->execute($self->{id});
	$self->{id} = undef;
}

}1;

=back

=head1 DEPENDENCIES

DBI, DBD::mysql, MySql database (see http://www.mysql.com)

=head1 AUTHOR

Kevin Hutchinson (kev@frontierworld.com)

=head1 COPYRIGHT

Copyright (c) 1999, FrontierWorld limited (UK).

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

==> p5-MyObject-1.2.tgz: lib/perl5/site_perl/5.005/i386-freebsd/MyObject.pm <==
#!/usr/bin/perl -w

=head1 NAME

MyObject - Store Perl objects in a MySql database

=head1 VERSION

This document refers to version 1.2 of MyObject, released Jul 12, 2000.
Download MyObject.pm from http://www.frontierworld.com/perl/MyObject.pm

=head1 SYNOPSIS

First create a table Foo in a MySql database Bar:

	create table Foo
	(
		id		INTEGER UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		name		VARCHAR(255) NOT NULL,
		email		VARCHAR(255),
		telephone	VARCHAR(255),
		timestamp	TIMESTAMP
	);

Then create a Perl program to access table Foo in database Bar as objects:

	package Foo;
	@ISA = qw(MyObject);
	use MyObject;

	package main;
	Foo->connect(database=>'Bar', user=>'fred', password=>'bloggs');
	Foo->fields( qw(name email telephone) );
	my $foo = Foo->new(name=>'kev', email=>'kev@frontierworld.com');
	$foo->insert();
	$foo->{telephone} = '+44 207 359 0985';
	$foo->update();
	$foo = Foo->select('name=?', 'kev');
	print $foo->{name} . ' has email address ' . $foo->{email} . '\n';
	$foo->delete();
	Foo->disconnect();

=head1 DESCRIPTION

MyObject stores objects in a MySql database. It should be inherited by a class
wishing to persist its objects in the database. The inheriting class should
correspond to a table of the same name (case-sensitive) in the database. This
table must have an auto_increment column called "id" as its primary key.

MyObject provides database connectivity via methods connect() and disconnect(),
object creation via methods new(), select() and next(), and object/database
syncronisation via methods insert(), update() and delete().

MyObject does not check whether objects have been modified in the database while
they were being modified in your Perl program. Nor does it support any locking
mechanisms. Basically it's designed to be fast, simple, easy and predictable.

=cut
package MyObject;
$VERSION = "1.0";

use strict;
use Carp;
use DBI;
{
	# Class static properties

	my $_Connection = undef;# A single shared database connection

	# Class static references to hashes keyed by class/table name

	my $_Connections = {};	# Database connections

	my $_Fields  = {};	# Fields to be syncronised with the database
	my $_Queries = {};	# User-defined queries for select() and next()

	my $_Selects = {};	# Standard select queries for row()
	my $_Inserts = {};	# Standard insert queries for insert()/update()
	my $_Updates = {};	# Standard update queries for update()
	my $_Deletes = {};	# Standard delete queries for delete()

	# Error messages

	my $_ERROR_FIELDS = "call class static method fields() first";

=head2 Class Methods

=over 4

=item connect(driver=>'mysql', database=>'dbname', user=>'username', password=>'pass', fields=>[qw(fields1 field2 field3)])

Connect to the database with optional connection details, and return the DBI
connection. The database driver always defaults to "mysql" unless environment
variable $DRIVER has been set. The database, user and password will default
to environment variables $DATABASE, $USER and $PASSWORD. If a field list is
not supplied, then a separate call to fields() must be made after connecting.

=cut
sub connect
{
	my ($self, %arg) = @_;
	my $class = ref($self) || $self;
	return if $_Connections->{$class};

	# Get the connection details

	my $driver	= $arg{driver}		|| $ENV{DRIVER} || "mysql";
	my $database	= $arg{database}	|| $ENV{DATABASE};
	my $user	= $arg{user}		|| $ENV{USER};
	my $password	= $arg{password}	|| $ENV{PASSWORD};

	# Connect to the database using a single shared connection

	$_Connection = DBI->connect("DBI:$driver:$database", $user, $password)
		unless $_Connection;

	$_Connections->{$class} = $_Connection;

	# Create standard queries for the database fields

	$class->fields(@{ $arg{fields} }) if $arg{fields};

	# Return the database connection

	return $_Connections->{$class}
			or croak("cannot connect to database: $DBI::errstr");
}

=item disconnect()

Disconnect from the database cleanly. You must call disconnect() before calling
connect() for a second time, otherwise connect() will merely return a handle to
the current database connection.

=cut
sub disconnect
{
	my ($self) = @_;
	my $class = ref($self) || $self;
	return unless $_Connections->{$class};

	# Finish user-defined queries

	foreach my $query (keys %{ $_Queries->{$class} })
	{
		$_Queries->{$class}->{$query}->finish();
	}

	# Finish standard queries

	$_Selects->{$class}->finish();
	$_Deletes->{$class}->finish();
	$_Inserts->{$class}->finish();
	$_Updates->{$class}->finish();

	# Close the database connection and disconnect if it's the last one

	$_Connections->{$class} = undef;

	my $connected = 0;
	foreach my $class (keys %{$_Connections})
	{
		$connected++ if $_Connections->{$class};
	}

	$_Connection->disconnect() unless $connected;
}

=item fields('field1', 'field2', ...)

Setup a list of fields to keep in sync with the database. Do not include the
"id" field or any timestamp fields in the list. Only include fields that you
want to keep syncronised with the database. Call fields() immediately after
calling connect() if you didn't provide a field list when you called connect().

=cut
sub fields
{
	my ($self, @fields) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} = [@fields];

	my $dbh = $_Connections->{$class} || $class->connect();

	# Prepare standard queries

	$_Selects->{$class} = $dbh->prepare("select * from $class where id = ?");
	$_Deletes->{$class} = $dbh->prepare("delete from $class where id = ?");

	my $field_list = "";
	my $value_list = "";
	my $update_list = "";
	foreach my $field (@fields)
	{
		$field_list .= ", " if $field_list;
		$field_list .= $field;

		$value_list .= ", " if $value_list;
		$value_list .= "?";

		$update_list .= ", " if $update_list;
		$update_list .= "$field = ?";
	}

	$_Inserts->{$class} = $dbh->prepare("insert into $class ($field_list) values ($value_list)");
	$_Updates->{$class} = $dbh->prepare("update $class set $update_list where id = ?");
}

=item new(field1=>'value1', field2=>'value2', ...)

Create a new object, setting any field values that were passed. This call does
not store the new object in the database - you should call insert() or update().

=cut
sub new
{
	my ($self, %arg) = @_;
	my $class = ref($self) || $self;
	bless { %arg }, $class;
}

=item row(id)

Get a table row as an object, by requesting a specific id. This is very useful
when one table has a relationship with another table, implemented via foreign
keys, and you wish to get the related object.

=cut
sub row
{
	my ($self, $id) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or croak($_ERROR_FIELDS);

	# Get an object with a specific id from the database

	$_Selects->{$class}->execute($id);
	$self = $_Selects->{$class}->fetchrow_hashref() || {};

	bless $self, $class;
}

=item sql('select * from table where field=?', 'value')

Create and execute a parameterised SQL statement. The format of the conditions
and arguments is similar to "printf", in that a "?" is used as a placeholder
for a corresponding argument. For example, if you use three question marks, you
should provide three arguments to instantiate them. Returns the query created.

=cut
sub sql
{
	my ($self, $sql, @args) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or croak($_ERROR_FIELDS);

	# Create a user-defined query unless it's cached

	my $dbh = $_Connections->{$class};
	$_Queries->{$class}->{$sql} ||= $dbh->prepare($sql);

	# Execute the query

	my $query = $_Queries->{$class}->{$sql};
	$query->execute(@args);

	return $query;
}

=item select('field=?', 'value')

Select the first object matching some conditions. The format of the conditions
and arguments is similar to "printf", in that a "?" is used as a placeholder
for a corresponding argument. For example, if you use three question marks, you
should provide three arguments to instantiate them. Use next() to iterate.

=cut
sub select
{
	my ($self, $conditions, @args) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or croak($_ERROR_FIELDS);
	$conditions = "1" unless $conditions;

	# Create a new object by executing the query

	my $query = $class->sql("select * from $class where $conditions", @args);
	$self = $query->fetchrow_hashref() || {};

	bless $self, $class;
}

=item next('field=?')

Select the next object matching some conditions. The conditions must exactly
match those originally provided in a select() call. When next() cannot return
the next object, it returns an empty object. A typical use for select() and
next() is in a for loop like this:

	my $obj;
	for (   $obj = Class->select("field=?",value);
		$obj->{id};
		$obj = Class->next("field=?"))
	{
		# Do something with $obj
	}

=cut
sub next
{
	my ($self, $conditions) = @_;
	my $class = ref($self) || $self;
	$_Fields->{$class} or croak($_ERROR_FIELDS);
	$conditions = "1" unless $conditions;

	# Get the next object from a user-defined query

	my $sql = "select * from $class where $conditions";
	my $query = $_Queries->{$class}->{$sql}
		or croak("select query not definied for $class: $conditions");

	$self = $query->fetchrow_hashref() || {};

	bless $self, $class;
}

=back

=head2 Object Methods

=over 4

=item insert()

Insert an object into the database and give the object a new id.

=cut
sub insert
{
	my ($self) = @_;
	$self->{id} = undef;
	$self->update();
}

=item update()

Update an object in the database, or insert it if the object has no id.

=cut
sub update
{
	my ($self) = @_;
	my $class = ref($self);
	$_Fields->{$class} or croak($_ERROR_FIELDS);

	# Get any object properties that are field values

	my @args = ();
	foreach my $field (@{ $_Fields->{$class} })
	{
		push @args, $self->{$field};
	}

	if ($self->{id})
	{
		# Update the object in the database

		push @args, $self->{id};
		$_Updates->{$class}->execute(@args);
	}
	else
	{
		# Insert the object into the database

		$_Inserts->{$class}->execute(@args);
		$self->{id} = $_Inserts->{$class}->{mysql_insertid};
	}
}

=item delete()

Delete an object from the database.

=cut
sub delete
{
	my ($self) = @_;
	return unless $self->{id};
	my $class = ref($self);
	$_Fields->{$class} or croak($_ERROR_FIELDS);

	# Delete an object from the database and give it an undefined id

	$_Deletes->{$class}->execute($self->{id});
	$self->{id} = undef;
}

}1;

=back

=head1 DEPENDENCIES

DBI, DBD::mysql, MySql database (see http://www.mysql.com)

=head1 AUTHOR

Kevin Hutchinson (kevin@frontierworld.com)

=head1 COPYRIGHT

Copyright (c) 1999, FrontierWorld limited (UK).

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
